// --------
//	main.c
// --------

#include "gba.h"
#include "font_lz.h"

#define FONT_TILESET		((u32*)VRAM)
#define	BG0_MAP				((u16*)0x0600c000)

extern void VBlankIntrWait(void);
extern void LZ77UnCompVram(void *Srcp, void *Destp);

extern void interruptHandler(void);

static void loadFont(void)
{
	LZ77UnCompVram((void *)font_LZ,(void *)FONT_TILESET);

	BG_PALETTE_MEM[1]  = RGB(16, 16, 16);
	BG_PALETTE_MEM[2]  = RGB(31, 31, 31);

	BG_PALETTE_MEM[17] = RGB(0,  16, 0);
	BG_PALETTE_MEM[18] = RGB(0,  31, 0);

} // of loadFont()

static void print(u8 x, u8 y, u16 pal, const char *string)
{
	u16 *map = BG0_MAP;
	
	map += (y * 32) + x;
	pal <<= 12;
	
	while(*string != '\0') *map++ = ((*string++) - 32) | pal;

} // of print()

void AgbMain(void)
{
	// setup our interrupt handler
	REG_INTERRUPT = (u32)interruptHandler;
	REG_DISPSTAT |= 0x8;
	REG_IE = INT_VBLANK;
	REG_IME = 1;
	
	loadFont();
	REG_BG0CNT = BGCNT_TILEMAP_C000 | BGCNT_TILEDATA_0000 | BGCNT_PRIORITY_0;
	REG_BG0VOFS = -4;
	REG_DISPCNT = DISP_MODE_0 | DISP_BG0_ON;

	print(4,  0,  0, "A BUTTON : ");
	print(4,  2,  0, "B BUTTON : ");
	print(6,  4,  0, "SELECT : ");
	print(7,  6,  0, "START : ");
	print(7,  8,  0, "RIGHT : ");
	print(8,  10, 0, "LEFT : ");
	print(10, 12, 0, "UP : ");
	print(8,  14, 0, "DOWN : ");
	print(4,  16, 0, "R BUTTON : ");
	print(4,  18, 0, "L BUTTON : ");

	for( ; ; )
	{
		u16 keys = ~REG_P1;
		int i;

		for(i = 0; i <= 9; i++)
		{
			if(keys & (1 << i))
				print(15, i << 1, 1, "PRESSED    ");

			else
				print(15, i << 1, 0, "NOT PRESSED");

		}

		VBlankIntrWait();	
	}

} // of AgbMain()
